function PS311_SchemeDIC_I(nu, Tfinal)
% Crank-Nicolson for u_t = u_{xx}.
% ____________________________________________________________________________
% 18.311 (Principles of Applied Mathematics). MIT, Spring 2012. R. R. Rosales.
%
% function PS311_SchemeDIC_I(nu, Tfinal)
%
% Finite Differences Numerical Scheme (demo).
% Equation. .................................................... u_t = u_{xx}.
% Notation.  ........... u(n, j) = solution at x = x_n = n*dx, t = t_j = j*dt.
% Scheme name. ..............................................  Crank-Nicolson.
% Scheme parameter. ............................................ nu = dt/dx^2.
% Scheme equation.
%                (1 + nu)*u(n, j+1) - 0.50*nu*(u(n+1, j+1) + u(n-1, j+1))
%              = (1 - nu)*u(n, j)   + 0.50*nu*(u(n+1, j)   + u(n-1, j)).
%
% Initial condition. .... Discontinuous saw-tooth: u0 = x - pi*(1-sign(pi-x)).
% Integration done for ....................................... 0 < t < Tfinal.
% Input Tfinal optional. ....................... Default value: Tfinal = 2.00.
%
% --- CODE WRITTEN SO REPLACING INITIAL CONDITIONS, ETC., IS EASY. Try it! ---
% ============================================================================

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Length = 2*pi;   if nargin < 2; Tfinal = 2.00; end;   Freeze = 250;
Npoint = 50:50:200;
Test   = -1;% On: Test = +1;   Off: Test = -1;
EQUATN = 'u_t = u_{xx}';
SCHEME = 'Crank-Nicolson';
NU     = 'dt/dx^2';
EQNtex = 'u_t = u_{xx}';
SCHtex = 'Crank-Nicolson';
%
% ============================================================================
% ------------------------- MEANING OF THE VARIABLES -------------------------
% Length = period in space for solution: 0 <= x <= Length.
% Tfinal = time for integration:         0 <= t <= Tfinal.
% Freeze = cut off on solution's growth. If the errors grow, when the solution
%          reaches the size specified by Freeze, any further growth is stopped
%          by scaling down to size Freeze at each time step.  This is to avoid
%          problems with plotting, e.g.: axis limits over-flows.
% Npoint = positive integer array with the number of points in the space grid.
%          Integrations are done with dx=Length/Npoint(n); n=1:length(Npoint).
% Test   = Used to turn on/off (Test=1/-1) a test of the linear solver for the
%          scheme implicit equations --- which are solved using FFT's.
%
% ---- STRINGS WITH TEXT TO BE USED  BY COMMAND LINE MESSAGES TO THE USER ----
% EQUATN = Equation solved.
% SCHEME = Name of the scheme used.
% NU     = Meaning of nu.
%
% ----------- STRINGS WITH LATEX TO BE USED FOR TITLES AND FIGURES -----------
% Strings with LaTeX to be used for titles and figures.
% EQNtex = Equation solved.
% SCHtex = Name of the scheme used.
%
% ----------- BELOW: %@ marks places where different schemes could differ.
%
% ============================================================================
% Command line messages generated by demo, stating what it does.
%
fprintf( '\n Finite Differences Numerical Scheme (demo).')
fprintf(['\n EQUATION: ', EQUATN, '.'])
fprintf(['\n SCHEME:   ', SCHEME, '.\n'])
fprintf( '\n Check the scheme for STABILITY: is this scheme stable?')
fprintf(['\n If so, for which values of nu = ', NU, '?\n']) %@
fprintf( '\n WARNING: even a stable scheme blows up if its parameters are')
fprintf( '\n          selected outside of their stable range.   Observing')
fprintf( '\n          blow up for just one parameter value is not enough.')
fprintf( '\n          You have to explore a reasonable parameter range to')
fprintf( '\n          conclude "probable" instability -- you cannot prove')
fprintf( '\n          with certainty things by numerical experimentation.\n')
fprintf( '\n WARNING. If instabilities arise:  Once the errors grow large,')
fprintf( '\n          further growth is prevented by scaling the solution,')
fprintf( '\n          at each time step, down to a maximum fixed size.')
fprintf(['\n          Currently this size is ', num2str(Freeze), '.\n \n'])
%
% ============================================================================
% Request the user to input parameters if not in the script call.           %@
%
if nargin < 1
   nu = input(['  Enter the value of nu = ', NU, ' = ']);
   if isempty(nu); nu = 1; end
   fprintf('\n')
end
if nu <= 0; nu = 1; end
%
% ============================================================================
% Define initial conditions (stored in u0) and integrate.
% Note that changing the initial conditions is very simple.
%
for n = 1:length(Npoint)
   N  = Npoint(n);  dx = Length/N;  x = Length*(1:N)/N;  dt = nu*dx^2; %@
   u0 = x - pi*(1-sign(pi-x)) + (rand(1, N) - 0.5)/10^4;
   U  = u0;% --- U will have the solution, one time level per row.
   T  =  0;% --- T will have the times computed.
   t  = dt;% --- t will have the time to be computed next.
   up = u0;% --- In each time step, up contains the solution at the current
   un = u0;%     time, and the solution at the next time goes into un.
   er =  0;% --- This is used by the linear solver testing step below.
   %
   while t <= Tfinal + dt
      %
      % SCHEME IMPLEMENTATION HERE. -------------------------------- %
      % The equation for un is
      % [SCH]                (1-0.5*nu)*un+0.5*nu*(Sp*un+Sm*un) =
      %                      (1+0.5*nu)*up-0.5*nu*(Sp*up+Sm*up); %@
      % where (Sp*v)(j) = v(j+1)
      % and   (Sm*v)(j) = v(j-1), for v periodic.
      % [SCH] is easily solved using FFT's.
      kappa = 2*pi*(0:N-1)/N;   s2 = (sin(kappa/2)).^2; %@
      G     = (1-2*nu*s2)./(1+2*nu*s2); %@
      un    = real(ifft(G.*fft(up)));
      %
      % --- TEST the linear solver. -------------------------------- %
      if Test > 0
        Spun = [un(2:N), un(1)];   Smun = [un(N), un(1:N-1)];
        Spup = [up(2:N), up(1)];   Smup = [up(N), up(1:N-1)];
        wn   = (1 + nu)*un - 0.5*nu*(Spun + Smun); %@
        wp   = (1 - nu)*up + 0.5*nu*(Spup + Smup); %@
        er   = max(max(abs(wn-wp)), er);
      end
      %
      % --- FREEZE ERROR GROWTH ONCE ERRORS GET LARGE ENOUGH. ------ %
      %     We do this by re-scaling solution to maximum allowed size.
      uM = max(abs(un));   if uM > Freeze; un = (Freeze/uM)*un; end
      %
      % --- STORE UPDATED SOLUTION AT NEW TIME. -------------------- %
      U = [U; un];   T = [T; t];   up = un;   t = t + dt;
   end
   if Test > 0; fprintf([' Error in solving linear equations <= ', ...
                         num2str(er), '\n']); end
   %
   % --- PLOT. ----------------------------------------------------- %
   zmax = max(max(U));   zmin = min(min(U));
   MakeTheSchemeFigure(n, N);   whitebg('k');
   mesh([0, x], T, [U(:, N), U]);
   hold on;   grid on;   rotate3d on
   axis([0 Length 0 (t-dt) zmin zmax])
   %
   % --- Abreviations used for labels and similar. ----------------- %
   LW  = 'LineWidth';    Pos = 'Position';              UN = 'Units';
   FW  = 'FontWeight';   HoA = 'HorizontalAlignment';   Lf = 'Left';
   FS  = 'FontSize';     Nor = 'Normalized';            
   FU  = 'FontUnits';    Str = 'String';
   Bf  = 'Bold';
   %
   % --- Pretty-up the plot. --------------------------------------- %
   %
   PSN = [0.1500 0.1500 0.7350 0.7050];
   set(gca, LW, 2, Pos, PSN, FW, Bf, FS, 16)
   colormap(hsv);  view(-40, 40); %@
   %
   St1 = NU; %@
   St2 = [' = ', num2str(nu, '%5.3f')]; %@
   St3 = 'dx';   St4 = ' = 2 \pi / N';
   St5 = 'N';    St6 = [' = ', num2str(N)];
   % =========================================================================
   text(Str,St1, UN,Nor, Pos,[0.780 1.140], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St2, UN,Nor, Pos,[0.930 1.140], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St3, UN,Nor, Pos,[0.780 1.080], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St4, UN,Nor, Pos,[0.930 1.080], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St5, UN,Nor, Pos,[0.780 1.020], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St6, UN,Nor, Pos,[0.930 1.020], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   % =========================================================================
   St7 = SCHtex;
   St8 = EQNtex;
   % =========================================================================
   text(Str,St7, UN,Nor, Pos,[-0.15 1.140], HoA,Lf, FW,Bf, FU,Nor, FS,0.06)
   text(Str,St8, UN,Nor, Pos,[-0.15 1.050], HoA,Lf, FW,Bf, FU,Nor, FS,0.06)
   % =========================================================================
   xlabel('{\bf  x}', FU, Nor, FS, 0.10)
   ylabel('{\bf  t}', FU, Nor, FS, 0.10)
   zlabel('{\bf u }', FU, Nor, FS, 0.10, HoA, 'Right', 'Rotation', 0)
end
if Test > 0; fprintf('\n'); end
fprintf(' Rotate 3D is on in the figures.\n')
fprintf(' Type "close all" to close the figures. \n \n')
% ============================================================================

function MakeTheSchemeFigure(n, N)
% ============================================================================
SSz = get(0, 'ScreenSize');
SW  = SSz(3);% ------------ Screen Width.
SH  = SSz(4);% ------------ Screen Height.
Sml = min(SW, SH);% ------- Screen minimum length.
% ------------------------- Lengths below in fractions of Sml.
dX = 0.03*n;% --- Figure window distance from screen right edge.
dY = 0.03*n;% --- Figure window distance from screen top   edge.
fW = 6.55*0.07;% --- Figure window width.
fH = 5.00*0.07;% --- Figure window height.
% ------------------------- Figure position.
POF = [SW - (fW + dX)*Sml, SH - (fH + dY)*Sml, fW*Sml, fH*Sml];
%
NOF = ['Solution with N = ', num2str(N), ' points in space.'];
%
figure('Name', NOF, 'NumberTitle', 'off', 'Position', POF)
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF